import mpi.*;

/**
 * Compile: javac -cp $MPJ_HOME/lib/mpj.jar:. Pi_MPJ.java 
 * Execute in multicore mode:
			 mpjrun.sh -np 4 Pi_MPJ
 * Execute in clustrer mode: 
			1)create a file name it
			  machines
			2) add name or IP of machines i.e.
			127.0.0.1
			3) start deamons
			mpjboot machines
			4) run program.. make sure machines file is in the same directory
			mpjrun.sh -np 4 Pi_MPJ
			5) stop daemons
			mpjhault machines
 */

public class Pi_MPJ {

	final static int N = 10000000; 	// Intervals.


	public static void main(String args[]) throws Exception {
		MPI.Init(args);

		long startTime = System.currentTimeMillis();
		int rank = MPI.COMM_WORLD.Rank(), P = MPI.COMM_WORLD.Size();
		double pi = 0.0;
		double h = 1.0 / (double)N, sum = 0.0;
		for (int i = rank + 1; i <= N; i += P) {
			double x = h * ((double)i - 0.5);
			 sum += (4.0 / (1.0 + x * x));
		} 
                if (rank > 0) {
			double[] sendBuf = new double[]{ sum };	// 1-element array containing sum
			 MPI.COMM_WORLD.Send(sendBuf, 0, 1, MPI.DOUBLE, 0, 0);
		} else {
			double[] recvBuf = new double[1];
			pi += sum;
			for (int src = 1; src < P; src++) {
				MPI.COMM_WORLD.Recv(recvBuf, 0, 1, MPI.DOUBLE, src, 0);
				pi += recvBuf[0];
			}
		}
		long endTime = System.currentTimeMillis();

		if (rank == 0) {
			System.out.println("Value of pi: " + pi * h);
			System.out.println("Calculated in " + (endTime - startTime) + " milliseconds");
		}
		MPI.Finalize();
	}
}


